import * as React from 'react'
import {
    Table,
    Row,
    Col,
    Button,
    Modal,
    Form,
    Select,
    message,
    Input,
} from 'antd'
import { FormItemProps } from 'antd/lib/form'
import { ColumnProps } from 'antd/lib/table'
import Upload from '../../components/upload'
import DetailModal from './detailModal'
import PreviewGroup from '../../components/image/previewGroup'
import {
    useAddDeviceType,
    useDeviceTypes,
    DeviceTypeDictionary,
    DeviceTypeRecordProps,
} from '../../services/DeviceTypes'
const columns: ColumnProps<DeviceTypeRecordProps>[] = [
    {
        title: '设备类型',
        dataIndex: 'type_name',
        key: 'type_name',
    },
    {
        title: '经办人',
        dataIndex: 'agent',
        key: 'agent',
    },
    {
        title: '经办人电话',
        dataIndex: 'agent_phone',
        key: 'agent_phone',
    },
    {
        title: '申请时间',
        dataIndex: 'created_time',
        key: 'created_time',
    },

    {
        title: '审核状态',
        dataIndex: 'status',
        key: 'status',
        render: (_, record) => {
            return DeviceTypeDictionary[record.status]
        },
    },
    {
        title: '审核时间',
        dataIndex: 'approve_time',
        key: 'approve_time',
    },
    {
        title: '审核回复',
        dataIndex: 'feedback',
        key: 'feedback',
        render: (dataValue) => {
            return dataValue || '-'
        },
    },
]

const warehousing: React.FC = () => {
    const [form] = Form.useForm()
    const [detailState, changeShowDetailState] = React.useState<{
        visible: boolean
        detailValues?: DeviceTypeRecordProps
    }>({
        visible: false,
    })
    const [state, setState] = React.useState({
        modalVisible: false,
    })
    const { state: addState, requestAddType } = useAddDeviceType()

    //可申请的设备类型列表
    const { state: deviceTypesState, requestTypes } = useDeviceTypes()
    // 已申请的设备类型列表
    const {
        state: hasDeviceTypes,
        requestTypes: requestHasDeviceTypes,
    } = useDeviceTypes()

    const getAppleyedDeviceTypes = () => {
        requestHasDeviceTypes({ list_type: 1 })
    }

    React.useEffect(() => {
        getAppleyedDeviceTypes()
    }, [])
    const handleSwitchModalVisible = () => {
        setState((preState) => ({
            ...preState,
            modalVisible: !preState.modalVisible,
        }))
    }
    React.useEffect(() => {
        if (state.modalVisible) {
            requestTypes({ list_type: 3 })
        }
    }, [state.modalVisible])

    const handleSubmit = () => {
        form.validateFields().then((data: any) => {
            requestAddType(data)
                .then(() => {
                    message.success('提交申报设备成功')
                    form.resetFields()
                    getAppleyedDeviceTypes()
                    handleSwitchModalVisible()
                })
                .catch((error) => {
                    message.error({ content: error.message })
                })
        })
    }
    const tableCoulms = React.useMemo(() => {
        return columns.concat({
            title: '操作',
            render: (_, record) => {
                return (
                    <Button
                        onClick={() => {
                            changeShowDetailState((preState) => ({
                                ...preState,
                                visible: true,
                                detailValues: {
                                    ...record,
                                    status: DeviceTypeDictionary[
                                        record.status
                                    ] as any,
                                },
                            }))
                        }}
                    >
                        详情
                    </Button>
                )
            },
        })
    }, [columns, changeShowDetailState])
    const accessFormData: FormItemProps[] = React.useMemo(() => {
        return [
            {
                label: '经办人',
                name: 'agent',
                required: true,
                children: <Input placeholder="请输入经办人"></Input>,
                rules: [{ required: true, message: '请输入经办人' }],
            },
            {
                label: '经办人电话',
                required: true,
                name: 'agent_phone',
                children: <Input placeholder="请输入经办人电话"></Input>,
                rules: [{ required: true, message: '请输入经办人电话' }],
            },
            {
                label: '设备类型',
                required: true,
                name: 'device_type',
                rules: [{ required: true, message: '请选择设备类型' }],
                children: (
                    <Select placeholder="请选择设备类型">
                        {deviceTypesState.dataSource.map((deviceType) => {
                            return (
                                <Select.Option
                                    key={deviceType.type_code}
                                    value={deviceType.type_code}
                                >
                                    {deviceType.type_name}
                                </Select.Option>
                            )
                        })}
                    </Select>
                ),
            },
            {
                label: '备注',
                required: true,
                name: 'comment',
                rules: [{ required: true, message: '请输入备注' }],
                children: (
                    <Input.TextArea
                        placeholder="请输入备注"
                        maxLength={150}
                    ></Input.TextArea>
                ),
            },
            {
                label: '附件',
                required: true,
                name: 'appendix',
                dependencies: ['device_type'],
                rules: [
                    ({ getFieldValue }) => {
                        return {
                            validator(rule, value) {
                                const device_type = getFieldValue('device_type')
                                if (
                                    !value &&
                                    deviceTypesState.dataSource.find(
                                        (type) => type.type_code === device_type
                                    )?.need_appendix
                                ) {
                                    return Promise.reject('请上传附件')
                                }
                                return Promise.resolve()
                            },
                        }
                    },
                ],
                children: <Upload></Upload>,
            },
        ]
    }, [deviceTypesState.dataSource])

    return (
        <>
            <Row justify="end" gutter={[0, 30]}>
                <Col>
                    <Button type="primary" onClick={handleSwitchModalVisible}>
                        申报
                    </Button>
                </Col>
            </Row>
            <Table
                rowKey={(record) => record.type_code}
                loading={hasDeviceTypes.loading}
                dataSource={hasDeviceTypes.dataSource}
                columns={tableCoulms}
                pagination={false}
            ></Table>
            {detailState.detailValues ? (
                <DetailModal
                    isModalVisible={detailState.visible}
                    values={detailState.detailValues}
                    onChangeVisible={() => {
                        changeShowDetailState((preState) => ({
                            ...preState,
                            visible: false,
                            detailValues: undefined,
                        }))
                    }}
                ></DetailModal>
            ) : null}

            <Modal
                onCancel={() => {
                    form.resetFields()
                    handleSwitchModalVisible()
                }}
                title="类型申请"
                visible={state.modalVisible}
                okText="确定"
                cancelText="取消"
                onOk={handleSubmit}
                okButtonProps={{ loading: addState.loading }}
            >
                <Form form={form} labelCol={{ span: 6 }}>
                    {accessFormData.map((formItem) => {
                        return (
                            <Form.Item
                                key={formItem.name as string}
                                {...formItem}
                            ></Form.Item>
                        )
                    })}
                </Form>
            </Modal>
        </>
    )
}
export default warehousing
