import * as React from 'react'
import { RouteComponentProps } from 'react-router-dom'
import {
    Row,
    Button,
    Table,
    Form,
    Input,
    Select,
    Col,
    Modal,
    message,
    Descriptions,
    Drawer,
    Badge,
    Typography,
} from 'antd'
import { PROJECT_INFOS, DEVICE_INFOS, REVIEW_INFOS } from './config'
import { useProjectList } from '../../services/project'
import { ColumnProps } from 'antd/lib/table'
import { FormItemProps } from 'antd/lib/form/FormItem'
import {
    useDustList,
    useInsertDevice,
    DustRecordProps,
    ApplyDustStatus,
    DeviceState,
} from '../../services/dust'

const { Text } = Typography

const { Option } = Select
const tableColumns: ColumnProps<DustRecordProps>[] = [
    { ...PROJECT_INFOS.name },
    { ...DEVICE_INFOS.name },
    { ...DEVICE_INFOS.sn },
    {
        ...DEVICE_INFOS.state,
        render: (_: string, record: DustRecordProps) => {
            const online = record.state === 1
            return (
                <>
                    <Badge status={online ? 'success' : 'error'}></Badge>
                    <Text type={online ? 'success' : 'secondary'}>
                        {DeviceState[record.state as 0 | 1]}
                    </Text>
                </>
            )
        },
    },
    { ...DEVICE_INFOS.key },
    { ...DEVICE_INFOS.create_time },
    {
        ...REVIEW_INFOS.status,
        render: (_: string, record: DustRecordProps) =>
            ApplyDustStatus[record.status],
    },
    { ...REVIEW_INFOS.approve_time },
].map((colums) => {
    return {
        title: colums.label,
        dataIndex: colums.key,
        key: colums.key,
        render: (colums as any).render,
    }
})

const DEFAULT_PAGE = 1

const dust: React.FC<RouteComponentProps> = () => {
    const [searchState, setSearchState] = React.useState<{
        page: number
        filter: string
    }>({
        page: DEFAULT_PAGE,
        filter: '',
    })
    const [deviceModalState, handeChangeDeviceModalState] = React.useState<{
        visible: boolean
        datas?: DustRecordProps
    }>({
        visible: false,
        datas: undefined,
    })
    const [form] = Form.useForm()
    const { state: listState, request: requestDustList } = useDustList()
    const { state: insertState, request: requestInsert } = useInsertDevice()
    const [projectListState, requestPorjects] = useProjectList()

    const [state, setState] = React.useState({
        modalVisible: false,
    })

    React.useEffect(() => {
        requestDustList(searchState)
    }, [searchState])

    const onSearch = (data: any) => {
        setSearchState((preState) => ({ ...preState, ...data, page: 1 }))
    }

    const handelSwitchModalVisible = React.useCallback(() => {
        setState((preState) => ({
            ...preState,
            modalVisible: !preState.modalVisible,
        }))
    }, [])
    React.useEffect(() => {
        if (state.modalVisible) {
            requestPorjects()
        }
    }, [state.modalVisible])

    const handleSubmit = React.useCallback(() => {
        form.validateFields().then((data: any) => {
            requestInsert(data)
                .then(() => {
                    handelSwitchModalVisible()
                    message.success('新增扬尘设备成功')
                    requestDustList(searchState)
                })
                .catch((error) => {
                    message.error(error.message)
                })
        })
    }, [form, requestInsert])
    const formInputs: FormItemProps[] = React.useMemo(() => {
        return [
            {
                ...DEVICE_INFOS.name,
                required: true,
                children: <Input placeholder="请输入设备型号"></Input>,
            },
            {
                ...DEVICE_INFOS.sn,
                required: true,
                children: (
                    <Input placeholder="供应商7位代码（个人中心）+7位扬尘序列号"></Input>
                ),
            },
            {
                ...DEVICE_INFOS.device_model,
                required: true,
                children: <Input placeholder="请输入设备型号"></Input>,
            },
            {
                ...DEVICE_INFOS.batch,
                required: true,
                children: <Input placeholder="请输入生产批次"></Input>,
            },
            {
                ...DEVICE_INFOS.manufacturer,
                required: true,
                children: <Input placeholder="请输入生产厂商"></Input>,
            },
            {
                key: 'project_id',
                label: '项目',
                required: true,
                children: (
                    <Select
                        placeholder="请选择项目"
                        loading={projectListState.loading}
                    >
                        {projectListState.dataSouce.map((project) => {
                            return (
                                <Option key={project.id} value={project.id}>
                                    {project.name}
                                </Option>
                            )
                        })}
                    </Select>
                ),
            },
            {
                ...DEVICE_INFOS.person,
                required: false,
                children: <Input placeholder="请输入联系人"></Input>,
            },
            {
                ...DEVICE_INFOS.phone,
                required: false,
                children: <Input placeholder="请输入联系电话"></Input>,
            },
            {
                ...DEVICE_INFOS.video_url,
                required: false,

                children: <Input placeholder="请输入视频地址"></Input>,
            },
            {
                ...DEVICE_INFOS.remark,
                required: false,
                children: (
                    <Input.TextArea
                        maxLength={512}
                        placeholder="请输入备注"
                    ></Input.TextArea>
                ),
            },
        ].map((element) => {
            return {
                name: element.key,
                label: element.label,
                children: element.children,
                required: element.required,
            }
        })
    }, [projectListState.dataSouce])
    const showDetailDatas = React.useMemo(() => {
        if (deviceModalState.datas) {
            const values = deviceModalState.datas as DustRecordProps & {
                [propIndex: string]: string
            }
            return [
                {
                    name: '项目信息',
                    key: 'project',
                    values: Object.values(PROJECT_INFOS).map((info) => {
                        return {
                            ...info,
                            value: values?.[info.key],
                        }
                    }),
                },
                {
                    name: '设备信息',
                    key: 'device',
                    values: Object.values(DEVICE_INFOS).map((info) => {
                        if (info.key === 'state') {
                            return {
                                ...info,
                                value: DeviceState[values?.[info.key]],
                            }
                        }
                        return {
                            ...info,
                            value: values?.[info.key],
                        }
                    }),
                },
                {
                    name: '审批信息',
                    key: 'review',
                    values: Object.values(REVIEW_INFOS).map((info) => {
                        if (info.key === 'status') {
                            return {
                                ...info,
                                value: ApplyDustStatus[values?.[info.key]],
                            }
                        }
                        return {
                            ...info,
                            value: values?.[info.key],
                        }
                    }),
                },
            ]
        }
        return []
    }, [deviceModalState.datas])
    return (
        <React.Fragment>
            <Row justify="space-between" style={{ marginBottom: 20 }}>
                <Col span={20}>
                    <Form
                        layout="inline"
                        labelCol={{ span: 6 }}
                        wrapperCol={{ span: 18 }}
                        onFinish={onSearch}
                    >
                        <Col span={10}>
                            <Form.Item
                                label="关键字"
                                name="filter"
                                colon={false}
                            >
                                <Input
                                    autoComplete="off"
                                    placeholder="设备唯一编码项目名称或安监备案号"
                                />
                            </Form.Item>
                        </Col>

                        <Col span={4}>
                            <Form.Item>
                                <Button type="primary" htmlType="submit">
                                    搜索
                                </Button>
                            </Form.Item>
                        </Col>
                    </Form>
                </Col>
                <Col span={4}>
                    <Button type="primary" onClick={handelSwitchModalVisible}>
                        添加
                    </Button>
                </Col>
            </Row>
            <Table
                className="equipment-table"
                rowKey={(record) => record.id}
                columns={tableColumns.concat({
                    title: '操作',
                    render: (_, record) => {
                        return (
                            <Button
                                onClick={() => {
                                    handeChangeDeviceModalState({
                                        visible: true,
                                        datas: record,
                                    })
                                }}
                            >
                                详情
                            </Button>
                        )
                    },
                })}
                dataSource={listState.dataSource}
                loading={listState.loading}
                pagination={{
                    hideOnSinglePage: true,
                    total: listState.total,
                    current: listState.page,
                    showSizeChanger: false,
                    onChange: (page) => {
                        setSearchState((preState) => ({ ...preState, page }))
                    },
                }}
            ></Table>
            <Modal
                destroyOnClose
                width={800}
                okText="确认"
                cancelText="取消"
                visible={state.modalVisible}
                title="添加扬尘设备"
                onCancel={handelSwitchModalVisible}
                onOk={handleSubmit}
                okButtonProps={{ loading: insertState.loading }}
            >
                <Form form={form}>
                    {formInputs.map((element) => {
                        return (
                            <Form.Item
                                rules={
                                    element.required
                                        ? [
                                              {
                                                  required: true,
                                                  message: `请输入${element.label}`,
                                              },
                                          ]
                                        : undefined
                                }
                                labelCol={{ span: 6 }}
                                key={element.name as string}
                                {...element}
                            ></Form.Item>
                        )
                    })}
                </Form>
            </Modal>
            <Drawer
                title="详情"
                maskClosable
                onClose={() =>
                    handeChangeDeviceModalState({
                        visible: false,
                        datas: undefined,
                    })
                }
                visible={deviceModalState.visible}
                width={1000}
            >
                {showDetailDatas.map((infos) => {
                    return (
                        <Descriptions
                            column={{ xxl: 2, xl: 2 }}
                            title={infos.name}
                            key={infos.key}
                        >
                            {infos.values.map((info) => {
                                return (
                                    <Descriptions.Item
                                        label={info.label}
                                        key={info.key}
                                    >
                                        {info.value}
                                    </Descriptions.Item>
                                )
                            })}
                        </Descriptions>
                    )
                })}
            </Drawer>
        </React.Fragment>
    )
}
export default dust
